package server

import (
	"encoding/gob"
	"time"
)

// ExpiringSession is a session with server-side expiration check.
// Session data is saved in signed, encrypted cookies in the
// browser. We'd like these cookies to expire when a certain amount of
// time passes, or when the user closes the browser. We trust the
// browser for the latter, but we enforce time-based expiration on the
// server.
type ExpiringSession struct {
	Expiry time.Time
}

func newExpiringSession(ttl time.Duration) *ExpiringSession {
	return &ExpiringSession{
		Expiry: time.Now().Add(ttl),
	}
}

// Valid returns true if the session has not expired yet.
func (e *ExpiringSession) Valid() bool {
	return e != nil && time.Now().Before(e.Expiry)
}

func init() {
	gob.Register(&ExpiringSession{})
}

// Returns the intersection of two string lists (in O(N^2) time).
func intersectGroups(a, b []string) []string {
	var out []string
	for _, aa := range a {
		for _, bb := range b {
			if aa == bb {
				out = append(out, aa)
				break
			}
		}
	}
	return out
}
